/*
    File : common.h
    Author : Nicolas De Coster (Vigon)
    Date : 2020-01-22
    Version : 0.1
    License : MIT
    Version notes :
        - corrected bug (& mask instead of &&)
*/

#include <avr/io.h>

#define MOTOR Y

#define UART_BAUDRATE 115200

#define PASTER(x,y) x ## y
#define CONCATENATE(x,y)  PASTER(x,y)

#define OUT_REG       PORTB.OUT
#define DIR_REG       PORTB.DIR
#define OUT_STEP_PIN  0
#define OUT_DIR_PIN   (OUT_STEP_PIN+1)
#define OUT_MASK      (0b11 << OUT_STEP_PIN)

#define STEP_SIGNAL_LENGTH 10

#define X_BIT_POS  0
#define Y_BIT_POS  2
#define Z_BIT_POS  4

#define DO_NOTHING 0b00
#define LEFT_STEP  0b01
#define RIGHT_STEP 0b11
#define ANSWER     0b10

#define BIT_POS    (CONCATENATE(MOTOR,_BIT_POS))

#define STEP_BIT   (1 << BIT_POS)
#define DIR_BIT    (1 << (BIT_POS+1))
#define ACT_MASK   (0b11 << BIT_POS)
#define ANSWER_XOR (ANSWER << BIT_POS)

void setup() {
  // initialize serial
  Serial.begin(UART_BAUDRATE);
  DIR_REG |= OUT_MASK; //set the leds
}

void loop() {
/* LED TOGGLING (testint purposes)
  OUT_REG |= OUT_MASK;
  delay(100);
  OUT_REG &= ~OUT_MASK;
  delay(100);
  //*/
}

void serialEvent() {
//*
  // get the new byte:
  byte shifted_action = ((byte)Serial.read()) & ACT_MASK; //get the action but keep it shifted (to avoid loosing time)
  if (shifted_action ^ ANSWER_XOR) { //check that the message for this motor isn't 'ANSWER'
    //set the two bits to the output "as is"
#if BIT_POS >= OUT_STEP_PIN
   OUT_REG |= (shifted_action >> (BIT_POS - OUT_STEP_PIN));
#else
   OUT_REG |= (shifted_action << (OUT_STEP_PIN - BIT_POS));
#endif
    //delayMicroseconds(STEP_SIGNAL_LENGTH);
    OUT_REG &= ~OUT_MASK; //set the two output pins low;
  }
  else {
    //TODO: ad an answer on UART line here (? how to answer ?)
  }
//*/  
}
