## esp32

** Hardware Requirements **

* [ hello.USB-serial.FT230X] (http://academy.cba.mit.edu/classes/embedded_programming/index.html)
* [ hello.ESP32-WROOM ] (http://academy.cba.mit.edu/classes/embedded_programming/index.html))

** Software Requirements **

*  [Expressif esp32/esp8266 Serial bootloader utility `esptool.py` ] (https://github.com/espressif/esptool)
*  [esp micropython firmware ] (https://micropython.org/download#esp32)
*  [esp webrepl client] (https://github.com/micropython/webrepl)
*  [Adafruit Micropython tool ampy] (https://github.com/scientifichackers/ampy)
*  [tshell ] (https://github.com/dhylands/rshell)
*  [Thonny ] (https://thonny.org/)
*  [Microwebserver] ()

** Reference Material **

*  [MicroPython Documentation] (https://docs.micropython.org/en/latest/index.html)
*  [esp32 Micropython Quickref] (https://docs.micropython.org/en/latest/esp32/quickref.html)



### Installaion

1.  You will require a USB-to-serial FTDI230X board/cable to connect the PC to the esp32 board.
2.  Determine the port on which you have connected the FTDI
	* MacOSX - Apple > About This Mac > System Report > USB or `ls /dev/tty*`
	* Linux - at the command prompt `dmesg | tail` and locate the /dev/ttySxxx port
	* Windows - Use the control Panel > Device Manager > USB to determine the COMxx port
3.  Install `esptool.py` using `pip3 install esptool`
4.  Download the [esp firmware] (https://micropython.org/download#esp32) ( version > 1.9.4), current version is 1.12
5.  Change the esp32-SWITCH into program mode
6.  Erase the esp32 before flashing the firmware, [explicit instructions] (https://docs.micropython.org/en/latest/esp32/tutorial/intro.html#getting-the-firmware)

	```
	esptool.py --port /dev/ttyUSB0 erase_flash
	## if you get lots of FAT non-formatted errors, then you have not erased the flash

	esptool.py --chip esp32 --port /dev/ttyUSB0 write_flash -z 0x1000 esp32-idf3-20200123-v1.12-76-gdccace6f3.bin
	# port = /dev/ttyUSB0
	# firmware = esp32-idf3-2020....

	```

7.  If successful, then you are ready to test the implementation using REPL.  I suggest using [Thonny] (https://thonny.org/), however any comms program e.g. picocom, screen would work.

	Change the esp32-SWITCH to the RUN mode, and press the reset button

	Connect to the `esp32` using

	```
	picocom  -b 115200 -p /dev/ttyUSB0
	```

	If successful you should see the '>>>' of the REPL prompt.


### Networking

The esp32 can run in a Station Mode or an Access-Point mode.  The configuration can be typed into the REPL for immediate results or saved in `boot,py` such that it is executed automatically upon reset

	```
	# Station mode
	# connect to access point

	import network

	ssid="Your SSID"
	passwd="your_password"

	station = network.WLAN(network.STA_IF)
	station.active(True)
	station.connect(ssid, passwd)

	# assumes network has a DHCP server
	```

Confirm by checking the port using REPL

	```
	station.ifconfig()
	station.connect('essid')
	```

### WebREPL

WebREPL is only available in firmwares >1.9.4.  It allows configuration of the esp32 over the network from a browser.

1.	Check that the firmware has the webrepl modules
	
	```
	>>> help( modules )
	...	webrepl ... webrepl_setup
	...
	```

2.	Configure the esp32 to run webrepl on startup

	```
	import webrepl_setup
	# follow instructions to setup webrepl
	# reset the esp32

	# enable webrepl
	import webrepl
	webrepl.start()

	# esp32 will report the IP address used
	# otherwise, obtain using
	station.ifconfig()

	```

3.	On another system, run the webrepl client using
	
	a.  Hosted client  - `http://micropython.org/webrepl/` ,  type in `<esp32-ip-address>:8266`

	b.  Download web client or Python  client `webrepl_cli.py`at [github] (https://github.com/micropython/webrepl)


### Running Microwebserver

We can make the esp32 a web server by running an implementation of [Microwebserver] (https://github.com/jczic/MicroWebSrv) or [Ver 2 improved] (https://github.com/jczic/MicroWebSrv2). 

A good starting point can be found at:

	a)	YouTube: [ESP32 Micropython Web Server] (https://youtu.be/xscBwC1SrF4) by Rdagger86.

	b)	[Rdagger86 Site] (https://www.rototron.info/raspberry-pi-esp32-micropython-web-server-tutorial/)

Summary of how to setup Microwebserver:

1.	Install `rshell`, this allows you to copy files/folders directly to the esp32, as well as run REPL

2.	Write the network initialisation commands (as above) and save it on the esp32 as `main.py`. This will allow the configuration to be automatically run when reset/restarted.

3.	Expand all MicroServer files into a folder e.g. MSvr,

	a)	delete files `boot.py`, `LICENSE.md`, `README.md`, `_config.yml` and `.git folder`

	b)	rename `main.py` to `start.py`

	c)	edit `start.py`, uncomment the `routeHandlers` at the start and replace the `sry = ...` line with`srv = MicroWebSrv(routeHandlers=routeHandlers, webPath='/www/')`

	d)	copy all files and folders to the esp32 root.

	e)	run the script:

		```
		import start
		```

